@@ -1,21 +1,21 @@
 /*
+ * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1999-2003  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
- * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
- * INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
- * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: server.c,v 1.339.2.8.4.3 2003/02/18 03:27:58 marka Exp $ */
+/* $Id: server.c,v 1.339.2.32 2004/11/30 00:32:56 marka Exp $ */
 
 #include <config.h>
 
@@ -26,6 +26,7 @@
 #include <isc/dir.h>
 #include <isc/entropy.h>
 #include <isc/file.h>
+#include <isc/hash.h>
 #include <isc/lex.h>
 #include <isc/print.h>
 #include <isc/resource.h>
@@ -100,6 +101,13 @@
 			fatal(msg, result);			  \
 	} while (0)						  \
 
+struct ns_dispatch {
+	isc_sockaddr_t			addr;
+	unsigned int			dispatchgen;
+	dns_dispatch_t			*dispatch;
+	ISC_LINK(struct ns_dispatch)	link;
+};
+
 static void
 fatal(const char *msg, isc_result_t result);
 
@@ -124,6 +132,9 @@ configure_zone(cfg_obj_t *config, cfg_obj_t *zconfig, cfg_obj_t *vconfig,
 	       isc_mem_t *mctx, dns_view_t *view,
 	       ns_aclconfctx_t *aclconf);
 
+static void
+end_reserved_dispatches(ns_server_t *server, isc_boolean_t all);
+
 /*
  * Configure a single view ACL at '*aclp'.  Get its configuration by
  * calling 'getvcacl' (for per-view configuration) and maybe 'getscacl'
@@ -163,7 +174,7 @@ configure_view_acl(cfg_obj_t *vconfig, cfg_obj_t *config,
 	return (result);
 }
 
-#ifdef ISC_RFC2335
+#ifdef ISC_RFC2535
 static isc_result_t
 configure_view_dnsseckey(cfg_obj_t *vconfig, cfg_obj_t *key,
 			 dns_keytable_t *keytable, isc_mem_t *mctx)
@@ -398,9 +409,24 @@ get_view_querysource_dispatch(cfg_obj_t **maps,
 				     1000, 32768, 16411, 16433,
 				     attrs, attrmask, &disp);
 	if (result != ISC_R_SUCCESS) {
+		isc_sockaddr_t any;
+		char buf[ISC_SOCKADDR_FORMATSIZE];
+
+		switch (af) {
+		case AF_INET:
+			isc_sockaddr_any(&any);
+			break;
+		case AF_INET6:
+			isc_sockaddr_any6(&any);
+			break;
+		}
+		if (isc_sockaddr_equal(&sa, &any))
+			return (ISC_R_SUCCESS);
+		isc_sockaddr_format(&sa, buf, sizeof(buf));
 		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
 			      NS_LOGMODULE_SERVER, ISC_LOG_ERROR,
-			      "could not get query source dispatcher");
+			      "could not get query source dispatcher (%s)",
+			      buf);
 		return (result);
 	}
 
@@ -641,14 +667,17 @@ configure_view(dns_view_t *view, cfg_obj_t *config, cfg_obj_t *vconfig,
 	 */
 	CHECK(get_view_querysource_dispatch(maps, AF_INET, &dispatch4));
 	CHECK(get_view_querysource_dispatch(maps, AF_INET6, &dispatch6));
+	if (dispatch4 == NULL && dispatch6 == NULL) {
+		UNEXPECTED_ERROR(__FILE__, __LINE__,
+				 "unable to obtain neither an IPv4 nor"
+				 " an IPv6 dispatch");
+		result = ISC_R_UNEXPECTED;
+		goto cleanup;
+	}
 	CHECK(dns_view_createresolver(view, ns_g_taskmgr, 31,
 				      ns_g_socketmgr, ns_g_timermgr,
 				      0, ns_g_dispatchmgr,
 				      dispatch4, dispatch6));
-	if (dispatch4 != NULL)
-		dns_dispatch_detach(&dispatch4);
-	if (dispatch6 != NULL)
-		dns_dispatch_detach(&dispatch6);
 
 	/*
 	 * Set resolver's lame-ttl.
@@ -812,12 +841,35 @@ configure_view(dns_view_t *view, cfg_obj_t *config, cfg_obj_t *vconfig,
 	CHECK(configure_view_acl(vconfig, config, "allow-query",
 				 actx, ns_g_mctx, &view->queryacl));
 
-	CHECK(configure_view_acl(vconfig, config, "allow-recursion",
-				 actx, ns_g_mctx, &view->recursionacl));
+	if (strcmp(view->name, "_bind") != 0)
+		CHECK(configure_view_acl(vconfig, config, "allow-recursion",
+					 actx, ns_g_mctx, &view->recursionacl));
 
 	CHECK(configure_view_acl(vconfig, config, "allow-v6-synthesis",
 				 actx, ns_g_mctx, &view->v6synthesisacl));
 
+	/*
+	 * Warning if both "recursion no;" and allow-recursion are active
+	 * except for "allow-recursion { none; };".
+	 */
+	if (!view->recursion && view->recursionacl != NULL &&
+	    (view->recursionacl->length != 1 ||
+	     view->recursionacl->elements[0].type != dns_aclelementtype_any ||
+	     view->recursionacl->elements[0].negative != ISC_TRUE)) {
+		const char *forview = " for view ";
+		const char *viewname = view->name;
+
+		if (!strcmp(view->name, "_bind") ||
+		    !strcmp(view->name, "_default")) {
+			forview = "";
+			viewname = "";
+		}
+		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
+			      NS_LOGMODULE_SERVER, ISC_LOG_WARNING,
+			      "both \"recursion no;\" and \"allow-recursion\" "
+			      "active%s%s", forview, viewname);
+	}
+
 	CHECK(configure_view_acl(vconfig, config, "sortlist",
 				 actx, ns_g_mctx, &view->sortlist));
 
@@ -850,9 +902,42 @@ configure_view(dns_view_t *view, cfg_obj_t *config, cfg_obj_t *vconfig,
 	if (view->maxncachettl > 7 * 24 * 3600)
 		view->maxncachettl = 7 * 24 * 3600;
 
+	obj = NULL;
+	result = ns_config_get(maps, "root-delegation-only", &obj);
+	if (result == ISC_R_SUCCESS) {
+		dns_view_setrootdelonly(view, ISC_TRUE);
+		if (!cfg_obj_isvoid(obj)) {
+			dns_fixedname_t fixed;
+			dns_name_t *name;
+			isc_buffer_t b;
+			char *str;
+			cfg_obj_t *exclude;
+
+			dns_fixedname_init(&fixed);
+			name = dns_fixedname_name(&fixed);
+			for (element = cfg_list_first(obj);
+			     element != NULL;
+			     element = cfg_list_next(element)) {
+				exclude = cfg_listelt_value(element);
+				str = cfg_obj_asstring(exclude);
+				isc_buffer_init(&b, str, strlen(str));
+				isc_buffer_add(&b, strlen(str));
+				CHECK(dns_name_fromtext(name, &b, dns_rootname,
+							ISC_FALSE, NULL));
+				CHECK(dns_view_excludedelegationonly(view,
+								     name));
+			}
+		}
+	} else
+		dns_view_setrootdelonly(view, ISC_FALSE);
+
 	result = ISC_R_SUCCESS;
 
  cleanup:
+	if (dispatch4 != NULL)
+		dns_dispatch_detach(&dispatch4);
+	if (dispatch6 != NULL)
+		dns_dispatch_detach(&dispatch6);
 	if (cmctx != NULL)
 		isc_mem_detach(&cmctx);
 
@@ -905,7 +990,14 @@ create_version_zone(cfg_obj_t **maps, dns_zonemgr_t *zmgr, dns_view_t *view) {
 	isc_region_t r;
 	size_t len;
 	dns_rdata_t rdata = DNS_RDATA_INIT;
-	static unsigned char origindata[] = "\007version\004bind";
+	static const unsigned char origindata[] = "\007version\004bind";
+	static const unsigned char soadata[] = "\007version\004bind\0"
+					 "\012hostmaster\007version\004bind\0"
+					 "\0\0\0\001"	/* serial */
+					 "\0\0\0\0"	/* refresh */
+					 "\0\0\0\0"	/* retry */
+					 "\0\0\0\0"	/* expire */
+					 "\0\0\0\0";	/* minimum */
 	dns_name_t origin;
 	cfg_obj_t *obj = NULL;
 	dns_acl_t *acl = NULL;
@@ -913,7 +1005,7 @@ create_version_zone(cfg_obj_t **maps, dns_zonemgr_t *zmgr, dns_view_t *view) {
 	dns_diff_init(ns_g_mctx, &diff);
 
 	dns_name_init(&origin, NULL);
-	r.base = origindata;
+	DE_CONST(origindata, r.base);
 	r.length = sizeof(origindata);
 	dns_name_fromregion(&origin, &r);
 
@@ -921,15 +1013,11 @@ create_version_zone(cfg_obj_t **maps, dns_zonemgr_t *zmgr, dns_view_t *view) {
 	INSIST(result == ISC_R_SUCCESS);
 	versiontext = cfg_obj_asstring(obj);
 	len = strlen(versiontext);
-	if (len > 255)
+	if (len > 255U)
 		len = 255; /* Silently truncate. */
 	buf[0] = len;
 	memcpy(buf + 1, versiontext, len);
 
-	r.base = buf;
-	r.length = 1 + len;
-	dns_rdata_fromregion(&rdata, dns_rdataclass_ch, dns_rdatatype_txt, &r);
-
 	CHECK(dns_zone_create(&zone, ns_g_mctx));
 	CHECK(dns_zone_setorigin(zone, &origin));
 	dns_zone_settype(zone, dns_zone_master);
@@ -947,9 +1035,35 @@ create_version_zone(cfg_obj_t **maps, dns_zonemgr_t *zmgr, dns_view_t *view) {
 
 	CHECK(dns_db_newversion(db, &dbver));
 
+	/* SOA record. */
+	tuple = NULL;
+	DE_CONST(soadata, r.base);
+	r.length = sizeof(soadata) - 1;
+	dns_rdata_fromregion(&rdata, dns_rdataclass_ch, dns_rdatatype_soa, &r);
 	CHECK(dns_difftuple_create(ns_g_mctx, DNS_DIFFOP_ADD, &origin,
 				   0, &rdata, &tuple));
 	dns_diff_append(&diff, &tuple);
+	dns_rdata_reset(&rdata);
+
+	/* NS record. */
+	tuple = NULL;
+	DE_CONST(origindata, r.base);
+	r.length = sizeof(origindata);
+	dns_rdata_fromregion(&rdata, dns_rdataclass_ch, dns_rdatatype_ns, &r);
+	CHECK(dns_difftuple_create(ns_g_mctx, DNS_DIFFOP_ADD, &origin,
+				   0, &rdata, &tuple));
+	dns_diff_append(&diff, &tuple);
+	dns_rdata_reset(&rdata);
+
+	/* TXT record. */
+	tuple = NULL;
+	r.base = buf;
+	r.length = 1 + len;
+	dns_rdata_fromregion(&rdata, dns_rdataclass_ch, dns_rdatatype_txt, &r);
+	CHECK(dns_difftuple_create(ns_g_mctx, DNS_DIFFOP_ADD, &origin,
+				   0, &rdata, &tuple));
+	dns_diff_append(&diff, &tuple);
+
 	CHECK(dns_diff_apply(&diff, db, dbver));
 
 	dns_db_closeversion(db, &dbver, ISC_TRUE);
@@ -957,7 +1071,7 @@ create_version_zone(cfg_obj_t **maps, dns_zonemgr_t *zmgr, dns_view_t *view) {
 	CHECK(dns_zone_replacedb(zone, db, ISC_FALSE));
 
 	CHECK(dns_view_addzone(view, zone));
-
+			
 	result = ISC_R_SUCCESS;
 
  cleanup:
@@ -985,10 +1099,17 @@ create_authors_zone(cfg_obj_t *options, dns_zonemgr_t *zmgr, dns_view_t *view)
 	dns_dbversion_t *dbver = NULL;
 	dns_difftuple_t *tuple;
 	dns_diff_t diff;
-	isc_constregion_t r;
-	isc_constregion_t cr;
+	isc_region_t r;
+	isc_region_t cr;
 	dns_rdata_t rdata = DNS_RDATA_INIT;
-	static const char origindata[] = "\007authors\004bind";
+	static const unsigned char origindata[] = "\007authors\004bind";
+	static const unsigned char soadata[] = "\007authors\004bind\0"
+					 "\012hostmaster\007authors\004bind\0"
+					 "\0\0\0\001"	/* serial */
+					 "\0\0\0\0"	/* refresh */
+					 "\0\0\0\0"	/* retry */
+					 "\0\0\0\0"	/* expire */
+					 "\0\0\0\0";	/* minimum */
 	dns_name_t origin;
 	int i;
 	static const char *authors[] = {
@@ -1019,9 +1140,9 @@ create_authors_zone(cfg_obj_t *options, dns_zonemgr_t *zmgr, dns_view_t *view)
 	dns_diff_init(ns_g_mctx, &diff);
 
 	dns_name_init(&origin, NULL);
-	r.base = origindata;
+	DE_CONST(origindata, r.base);
 	r.length = sizeof(origindata);
-	dns_name_fromregion(&origin, (isc_region_t *)&r);
+	dns_name_fromregion(&origin, &r);
 
 	CHECK(dns_zone_create(&zone, ns_g_mctx));
 	CHECK(dns_zone_setorigin(zone, &origin));
@@ -1040,12 +1161,33 @@ create_authors_zone(cfg_obj_t *options, dns_zonemgr_t *zmgr, dns_view_t *view)
 
 	CHECK(dns_db_newversion(db, &dbver));
 
+	/* SOA record. */
+	tuple = NULL;
+	DE_CONST(soadata, r.base);
+	r.length = sizeof(soadata) - 1;
+	dns_rdata_fromregion(&rdata, dns_rdataclass_ch, dns_rdatatype_soa, &r);
+	CHECK(dns_difftuple_create(ns_g_mctx, DNS_DIFFOP_ADD, &origin,
+				   0, &rdata, &tuple));
+	dns_diff_append(&diff, &tuple);
+	dns_rdata_reset(&rdata);
+
+	/* NS record. */
+	tuple = NULL;
+	DE_CONST(origindata, r.base);
+	r.length = sizeof(origindata);
+	dns_rdata_fromregion(&rdata, dns_rdataclass_ch, dns_rdatatype_ns, &r);
+	CHECK(dns_difftuple_create(ns_g_mctx, DNS_DIFFOP_ADD, &origin,
+				   0, &rdata, &tuple));
+	dns_diff_append(&diff, &tuple);
+	dns_rdata_reset(&rdata);
+
+	/* TXT records. */
 	for (i = 0; authors[i] != NULL; i++) {
-		cr.base = authors[i];
+		DE_CONST(authors[i], cr.base);
 		cr.length = strlen(authors[i]);
-		INSIST(cr.length == ((const unsigned char *)cr.base)[0] + 1U);
+		INSIST(cr.length == cr.base[0] + 1U);
 		dns_rdata_fromregion(&rdata, dns_rdataclass_ch,
-				     dns_rdatatype_txt, (isc_region_t *)&cr);
+				     dns_rdatatype_txt, &cr);
 		tuple = NULL;
 		CHECK(dns_difftuple_create(ns_g_mctx, DNS_DIFFOP_ADD, &origin,
 					   0, &rdata, &tuple));
@@ -1248,7 +1390,9 @@ configure_zone(cfg_obj_t *config, cfg_obj_t *zconfig, cfg_obj_t *vconfig,
 	cfg_obj_t *typeobj = NULL;
 	cfg_obj_t *forwarders = NULL;
 	cfg_obj_t *forwardtype = NULL;
+	cfg_obj_t *only = NULL;
 	isc_result_t result;
+	isc_result_t tresult;
 	isc_buffer_t buffer;
 	dns_fixedname_t fixorigin;
 	dns_name_t *origin;
@@ -1314,14 +1458,25 @@ configure_zone(cfg_obj_t *config, cfg_obj_t *zconfig, cfg_obj_t *vconfig,
 		}
 		if (dns_name_equal(origin, dns_rootname)) {
 			char *hintsfile = cfg_obj_asstring(fileobj);
+
 			result = configure_hints(view, hintsfile);
-			if (result != ISC_R_SUCCESS)
+			if (result != ISC_R_SUCCESS) {
 				isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
 					      NS_LOGMODULE_SERVER,
 					      ISC_LOG_ERROR,
 					      "could not configure root hints "
 					      "from '%s': %s", hintsfile,
 					      isc_result_totext(result));
+				goto cleanup;
+			}
+			/*
+			 * Hint zones may also refer to delegation only points.
+			 */
+			only = NULL;
+			tresult = cfg_map_get(zoptions, "delegation-only",
+					      &only);
+			if (tresult == ISC_R_SUCCESS && cfg_obj_asboolean(only))
+				CHECK(dns_view_adddelegationonly(view, origin));
 		} else {
 			isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
 				      NS_LOGMODULE_SERVER, ISC_LOG_WARNING,
@@ -1348,6 +1503,14 @@ configure_zone(cfg_obj_t *config, cfg_obj_t *zconfig, cfg_obj_t *vconfig,
 		goto cleanup;
 	}
 
+	/*
+	 * "delegation-only zones" aren't zones either.
+	 */
+	if (strcasecmp(ztypestr, "delegation-only") == 0) {
+		result = dns_view_adddelegationonly(view, origin);
+		goto cleanup;
+	}
+
 	/*
 	 * Check for duplicates in the new zone table.
 	 */
@@ -1415,6 +1578,16 @@ configure_zone(cfg_obj_t *config, cfg_obj_t *zconfig, cfg_obj_t *vconfig,
 					forwardtype));
 	}
 
+	/*
+	 * Stub and forward zones may also refer to delegation only points.
+	 */
+	only = NULL;
+	if (cfg_map_get(zoptions, "delegation-only", &only) == ISC_R_SUCCESS)
+	{
+		if (cfg_obj_asboolean(only))
+			CHECK(dns_view_adddelegationonly(view, origin));
+	}
+
 	/*
 	 * Configure the zone.
 	 */
@@ -1623,8 +1796,6 @@ load_configuration(const char *filename, ns_server_t *server,
 	dns_viewlist_t viewlist;
 	dns_viewlist_t tmpviewlist;
 	ns_aclconfctx_t aclconfctx;
-	dns_dispatch_t *dispatchv4 = NULL;
-	dns_dispatch_t *dispatchv6 = NULL;
 	isc_uint32_t interface_interval;
 	isc_uint32_t heartbeat_interval;
 	in_port_t listen_port;
@@ -2101,11 +2272,6 @@ load_configuration(const char *filename, ns_server_t *server,
 
 	}
 
-	if (dispatchv4 != NULL)
-		dns_dispatch_detach(&dispatchv4);
-	if (dispatchv6 != NULL)
-		dns_dispatch_detach(&dispatchv6);
-
 	/* Relinquish exclusive access to configuration data. */
 	isc_task_endexclusive(server->task);
 
@@ -2162,6 +2328,12 @@ load_new_zones(ns_server_t *server, isc_boolean_t stop) {
 	{
 		CHECK(dns_view_loadnew(view, stop));
 	}
+	/*
+	 * Force zone maintenance.  Do this after loading
+	 * so that we know when we need to force AXFR of
+	 * slave zones whose master files are missing.
+	 */
+	CHECK(dns_zonemgr_forcemaint(server->zonemgr));
  cleanup:
 	isc_task_endexclusive(server->task);	
 	return (result);
@@ -2172,7 +2344,7 @@ run_server(isc_task_t *task, isc_event_t *event) {
 	isc_result_t result;
 	ns_server_t *server = (ns_server_t *)event->ev_arg;
 
-	UNUSED(task);
+	INSIST(task == server->task);
 
 	isc_event_free(&event);
 
@@ -2208,11 +2380,13 @@ run_server(isc_task_t *task, isc_event_t *event) {
 		CHECKFATAL(load_configuration(ns_g_conffile, server, ISC_TRUE),
 			   "loading configuration");
 
-	CHECKFATAL(load_zones(server, ISC_FALSE),
-		   "loading zones");
+	isc_hash_init();
 
+	CHECKFATAL(load_zones(server, ISC_FALSE), "loading zones");
+
+	ns_os_started();
 	isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL, NS_LOGMODULE_SERVER,
-		      ISC_LOG_INFO, "running");
+		      ISC_LOG_NOTICE, "running");
 }
 
 void 
@@ -2241,6 +2415,7 @@ shutdown_server(isc_task_t *task, isc_event_t *event) {
 		      flush ? ": flushing changes" : "");
 
 	ns_controls_shutdown(server->controls);
+	end_reserved_dispatches(server, ISC_TRUE);
 
 	cfg_obj_destroy(ns_g_parser, &ns_g_config);
 	cfg_parser_destroy(&ns_g_parser);
@@ -2269,6 +2444,8 @@ shutdown_server(isc_task_t *task, isc_event_t *event) {
 	if (server->blackholeacl != NULL)
 		dns_acl_detach(&server->blackholeacl);
 
+	dns_db_detach(&server->in_roothints);
+
 	isc_task_endexclusive(server->task);
 
 	isc_task_detach(&server->task);
@@ -2369,6 +2546,8 @@ ns_server_create(isc_mem_t *mctx, ns_server_t **serverp) {
 	server->controls = NULL;
 	CHECKFATAL(ns_controls_create(server, &server->controls),
 		   "ns_controls_create");
+	server->dispatchgen = 0;
+	ISC_LIST_INIT(server->dispatches);
 
 	server->magic = NS_SERVER_MAGIC;
 	*serverp = server;
@@ -2397,8 +2576,6 @@ ns_server_destroy(ns_server_t **serverp) {
 
 	INSIST(ISC_LIST_EMPTY(server->viewlist));
 
-	dns_db_detach(&server->in_roothints);
-
 	dns_aclenv_destroy(&server->aclenv);
 
 	isc_quota_destroy(&server->recursionquota);
@@ -2420,14 +2597,118 @@ fatal(const char *msg, isc_result_t result) {
 	exit(1);
 }
 
+static void
+start_reserved_dispatches(ns_server_t *server) {
+
+	REQUIRE(NS_SERVER_VALID(server));
+
+	server->dispatchgen++;
+}
+
+static void
+end_reserved_dispatches(ns_server_t *server, isc_boolean_t all) {
+	ns_dispatch_t *dispatch, *nextdispatch;
+
+	REQUIRE(NS_SERVER_VALID(server));
+
+	for (dispatch = ISC_LIST_HEAD(server->dispatches);
+	     dispatch != NULL;
+	     dispatch = nextdispatch) {
+		nextdispatch = ISC_LIST_NEXT(dispatch, link);
+		if (!all && server->dispatchgen == dispatch-> dispatchgen)
+			continue;
+		ISC_LIST_UNLINK(server->dispatches, dispatch, link);
+		dns_dispatch_detach(&dispatch->dispatch);
+		isc_mem_put(server->mctx, dispatch, sizeof(*dispatch));
+	}
+}
+
+void
+ns_add_reserved_dispatch(ns_server_t *server, isc_sockaddr_t *addr) {
+	ns_dispatch_t *dispatch;
+	in_port_t port;
+	char addrbuf[ISC_SOCKADDR_FORMATSIZE];
+	isc_result_t result;
+	unsigned int attrs, attrmask;
+
+	REQUIRE(NS_SERVER_VALID(server));
+
+	port = isc_sockaddr_getport(addr);
+	if (port == 0 || port >= 1024)
+		return;
+
+	for (dispatch = ISC_LIST_HEAD(server->dispatches);
+	     dispatch != NULL;
+	     dispatch = ISC_LIST_NEXT(dispatch, link)) {
+		if (isc_sockaddr_equal(&dispatch->addr, addr))
+			break;
+	}
+	if (dispatch != NULL) {
+		dispatch->dispatchgen = server->dispatchgen;
+		return;
+	}
+
+	dispatch = isc_mem_get(server->mctx, sizeof(*dispatch));
+	if (dispatch == NULL) {
+		result = ISC_R_NOMEMORY;
+		goto cleanup;
+	}
+
+	dispatch->addr = *addr;
+	dispatch->dispatchgen = server->dispatchgen;
+	dispatch->dispatch = NULL;
+
+        attrs = 0;
+        attrs |= DNS_DISPATCHATTR_UDP;
+        switch (isc_sockaddr_pf(addr)) {
+        case AF_INET:
+                attrs |= DNS_DISPATCHATTR_IPV4;
+                break;
+        case AF_INET6:
+                attrs |= DNS_DISPATCHATTR_IPV6;
+                break;
+	default:
+		result = ISC_R_NOTIMPLEMENTED;
+		goto cleanup;
+        }
+        attrmask = 0;
+        attrmask |= DNS_DISPATCHATTR_UDP;
+        attrmask |= DNS_DISPATCHATTR_TCP;
+        attrmask |= DNS_DISPATCHATTR_IPV4;
+        attrmask |= DNS_DISPATCHATTR_IPV6;
+
+	result = dns_dispatch_getudp(ns_g_dispatchmgr, ns_g_socketmgr,
+                                     ns_g_taskmgr, &dispatch->addr, 4096,
+                                     1000, 32768, 16411, 16433,
+                                     attrs, attrmask, &dispatch->dispatch); 
+	if (result != ISC_R_SUCCESS)
+		goto cleanup;
+
+	ISC_LIST_INITANDPREPEND(server->dispatches, dispatch, link);
+
+	return;
+
+ cleanup:
+	if (dispatch != NULL)
+		isc_mem_put(server->mctx, dispatch, sizeof(*dispatch));
+	isc_sockaddr_format(addr, addrbuf, sizeof(addrbuf));
+	isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
+		      NS_LOGMODULE_SERVER, ISC_LOG_WARNING,
+		      "unable to create dispatch for reserved port %s: %s",
+		      addrbuf, isc_result_totext(result));
+}
+
+
 static isc_result_t
 loadconfig(ns_server_t *server) {
 	isc_result_t result;
+	start_reserved_dispatches(server);
 	result = load_configuration(ns_g_lwresdonly ?
 				    lwresd_g_conffile : ns_g_conffile,
-				    server,
-				    ISC_FALSE);
-	if (result != ISC_R_SUCCESS)
+				    server, ISC_FALSE);
+	if (result == ISC_R_SUCCESS)
+		end_reserved_dispatches(server, ISC_FALSE);
+	else
 		isc_log_write(ns_g_lctx, NS_LOGCATEGORY_GENERAL,
 			      NS_LOGMODULE_SERVER, ISC_LOG_ERROR,
 			      "reloading configuration failed: %s",
@@ -2745,7 +3026,7 @@ ns_server_dumpstats(ns_server_t *server) {
 	fprintf(fp, "+++ Statistics Dump +++ (%lu)\n", (unsigned long)now);
 	
 	for (i = 0; i < ncounters; i++)
-		fprintf(fp, "%s %" ISC_PRINT_QUADFORMAT "d\n",
+		fprintf(fp, "%s %" ISC_PRINT_QUADFORMAT "u\n",
 			dns_statscounter_names[i],
 			server->querystats[i]);
 	
@@ -2766,7 +3047,7 @@ ns_server_dumpstats(ns_server_t *server) {
 			viewname = view->name;
 			for (i = 0; i < ncounters; i++) {
 				fprintf(fp, "%s %" ISC_PRINT_QUADFORMAT
-					"d %s",
+					"u %s",
 					dns_statscounter_names[i],
 					zonestats[i],
 					zonename);
@@ -2891,11 +3172,11 @@ ns_server_status(ns_server_t *server, isc_buffer_t *text) {
 					  DNS_ZONESTATE_SOAQUERY);
 	n = snprintf((char *)isc_buffer_used(text),
 		     isc_buffer_availablelength(text),
-		     "number of zones: %d\n"
+		     "number of zones: %u\n"
 		     "debug level: %d\n"
-		     "xfers running: %d\n"
-		     "xfers deferred: %d\n"
-		     "soa queries in progress: %d\n"
+		     "xfers running: %u\n"
+		     "xfers deferred: %u\n"
+		     "soa queries in progress: %u\n"
 		     "query logging is %s\n"
 		     "server is up and running",
 		     zonecount, ns_g_debuglevel, xferrunning, xferdeferred,
