@@ -1,21 +1,21 @@
 /*
+ * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
  * Copyright (C) 1998-2001  Internet Software Consortium.
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM
- * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
- * INTERNET SOFTWARE CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
- * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
- * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
- * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
+ * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
+ * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
+ * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: txt_16.c,v 1.32.4.1 2001/01/09 22:47:41 bwelling Exp $ */
+/* $Id: txt_16.c,v 1.37.2.1 2004/03/09 06:11:34 marka Exp $ */
 
 /* Reviewed: Thu Mar 16 15:40:00 PST 2000 by bwelling */
 
@@ -27,25 +27,30 @@
 static inline isc_result_t
 fromtext_txt(ARGS_FROMTEXT) {
 	isc_token_t token;
-
-	UNUSED(rdclass);
-	UNUSED(origin);
-	UNUSED(downcase);
+	int strings;
 
 	REQUIRE(type == 16);
 
-	for(;;) {
+	UNUSED(type);
+	UNUSED(rdclass);
+	UNUSED(origin);
+	UNUSED(downcase);
+	UNUSED(callbacks);
+
+	strings = 0;
+	for (;;) {
 		RETERR(isc_lex_getmastertoken(lexer, &token,
 					      isc_tokentype_qstring,
 					      ISC_TRUE));
 		if (token.type != isc_tokentype_qstring &&
 		    token.type != isc_tokentype_string)
 			break;
-		RETERR(txt_fromtext(&token.value.as_textregion, target));
+		RETTOK(txt_fromtext(&token.value.as_textregion, target));
+		strings++;
 	}
 	/* Let upper layer handle eol/eof. */
 	isc_lex_ungettoken(lexer, &token);
-	return (ISC_R_SUCCESS);
+	return (strings == 0 ? ISC_R_UNEXPECTEDEND : ISC_R_SUCCESS);
 }
 
 static inline isc_result_t
@@ -71,17 +76,18 @@ static inline isc_result_t
 fromwire_txt(ARGS_FROMWIRE) {
 	isc_result_t result;
 
+	REQUIRE(type == 16);
+
+	UNUSED(type);
 	UNUSED(dctx);
 	UNUSED(rdclass);
 	UNUSED(downcase);
 
-	REQUIRE(type == 16);
-
-	while (!buffer_empty(source)) {
+	do {
 		result = txt_fromwire(source, target);
 		if (result != ISC_R_SUCCESS)
 			return (result);
-	}
+	} while (!buffer_empty(source));
 	return (ISC_R_SUCCESS);
 }
 
@@ -126,9 +132,9 @@ fromstruct_txt(ARGS_FROMSTRUCT) {
 	REQUIRE(source != NULL);
 	REQUIRE(txt->common.rdtype == type);
 	REQUIRE(txt->common.rdclass == rdclass);
-	REQUIRE((txt->txt == NULL && txt->txt_len == 0) ||
-		(txt->txt != NULL && txt->txt_len != 0));
+	REQUIRE(txt->txt != NULL && txt->txt_len != 0);
 
+	UNUSED(type);
 	UNUSED(rdclass);
 
 	region.base = txt->txt;
@@ -158,12 +164,9 @@ tostruct_txt(ARGS_TOSTRUCT) {
 
 	dns_rdata_toregion(rdata, &r);
 	txt->txt_len = r.length;
-	if (txt->txt_len != 0) {
-		txt->txt = mem_maybedup(mctx, r.base, r.length);
-		if (txt->txt == NULL)
-			return (ISC_R_NOMEMORY);
-	} else
-		txt->txt = NULL;
+	txt->txt = mem_maybedup(mctx, r.base, r.length);
+	if (txt->txt == NULL)
+		return (ISC_R_NOMEMORY);
 
 	txt->offset = 0;
 	txt->mctx = mctx;
