@@ -1,44 +1,34 @@
 /*
- * Copyright (C) 1999, 2000  Internet Software Consortium.
- * 
- * This code is derived from software contributed to Internet Software
- * Consortium by Berkeley Software Design, Inc.
- * 
+ * Copyright (C) 2004  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) 1999-2001  Internet Software Consortium.
+ *
+ * This code is derived from software contributed to ISC by
+ * Berkeley Software Design, Inc.
+ *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
  * copyright notice and this permission notice appear in all copies.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM AND
- * BERKELEY SOFTWARE DESIGN, INC DISCLAIM ALL WARRANTIES WITH REGARD TO
- * THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE CONSORTIUM OR BERKELEY
- * SOFTWARE DESIGN, INC BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR
- * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
- * USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
- * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND ISC AND BERKELEY SOFTWARE DESIGN, INC.
+ * DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE
+ * FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
+ * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: getaddrinfo.c,v 1.23.2.5 2000/10/19 23:57:20 gson Exp $ */
+/* $Id: getaddrinfo.c,v 1.41.2.1 2004/03/09 06:12:33 marka Exp $ */
 
 #include <config.h>
 
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-
-#include <netinet/in.h>
-
-#include <arpa/nameser.h>
-#include <arpa/inet.h>
-
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
 
 #include <lwres/lwres.h>
 #include <lwres/net.h>
-#include <lwres/netdb.h>	/* XXX #include <netdb.h> */
+#include <lwres/netdb.h>
 
 #define SA(addr)	((struct sockaddr *)(addr))
 #define SIN(addr)	((struct sockaddr_in *)(addr))
@@ -83,7 +73,7 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 
 	proto = NULL;
 	if (hints != NULL) {
-		if (hints->ai_flags & ~(ISC_AI_MASK))
+		if ((hints->ai_flags & ~(ISC_AI_MASK)) != 0)
 			return (EAI_BADFLAGS);
 		if (hints->ai_addrlen || hints->ai_canonname ||
 		    hints->ai_addr || hints->ai_next) {
@@ -97,27 +87,42 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 		switch (family) {
 		case AF_UNSPEC:
 			switch (hints->ai_socktype) {
-			case SOCK_STREAM:	proto = "tcp"; break;
-			case SOCK_DGRAM:	proto = "udp"; break;
+			case SOCK_STREAM:
+				proto = "tcp";
+				break;
+			case SOCK_DGRAM:
+				proto = "udp";
+				break;
 			}
 			break;
 		case AF_INET:
 		case AF_INET6:
 			switch (hints->ai_socktype) {
-			case 0:			break;
-			case SOCK_STREAM:	proto = "tcp"; break;
-			case SOCK_DGRAM:	proto = "udp"; break;
-			case SOCK_RAW:		break;
-			default:		return (EAI_SOCKTYPE);
+			case 0:
+				break;
+			case SOCK_STREAM:
+				proto = "tcp";
+				break;
+			case SOCK_DGRAM:
+				proto = "udp";
+				break;
+			case SOCK_RAW:
+				break;
+			default:
+				return (EAI_SOCKTYPE);
 			}
 			break;
 #ifdef	AF_LOCAL
 		case AF_LOCAL:
 			switch (hints->ai_socktype) {
-			case 0:			break;
-			case SOCK_STREAM:	break;
-			case SOCK_DGRAM:	break;
-			default:		return (EAI_SOCKTYPE);
+			case 0:
+				break;
+			case SOCK_STREAM:
+				break;
+			case SOCK_DGRAM:
+				break;
+			default:
+				return (EAI_SOCKTYPE);
 			}
 			break;
 #endif
@@ -138,11 +143,11 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 	 * hostname/servname is '/'.
 	 */
 
-	if (hostname &&
+	if (hostname != NULL &&
 	    (family == AF_LOCAL || (family == 0 && *hostname == '/')))
 		return (get_local(hostname, socktype, res));
 
-	if (servname &&
+	if (servname != NULL &&
 	    (family == AF_LOCAL || (family == 0 && *servname == '/')))
 		return (get_local(servname, socktype, res));
 #endif
@@ -157,7 +162,7 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 	 * requested.  If the socket type wasn't specified, then
 	 * try and figure it out.
 	 */
-	if (servname) {
+	if (servname != NULL) {
 		char *e;
 
 		port = strtol(servname, &e, 10);
@@ -166,7 +171,7 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 				return (EAI_SOCKTYPE);
 			if (port < 0 || port > 65535)
 				return (EAI_SERVICE);
-			port = htons(port);
+			port = htons((unsigned short) port);
 		} else {
 			sp = getservbyname(servname, proto);
 			if (sp == NULL)
@@ -217,7 +222,7 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 
 	/*
 	 * If the family isn't specified or AI_NUMERICHOST specified,
-	 * check first to see if it * is a numeric address.
+	 * check first to see if it is a numeric address.
 	 * Though the gethostbyname2() routine
 	 * will recognize numeric addresses, it will only recognize
 	 * the format that it is being called for.  Thus, a numeric
@@ -265,7 +270,9 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 			scopeid = 0;
 #endif
 
-               if (lwres_net_pton(AF_INET, hostname, (struct in_addr *)abuf)) {
+               if (lwres_net_pton(AF_INET, hostname, (struct in_addr *)abuf)
+		   == 1)
+	       {
 			if (family == AF_INET6) {
 				/*
 				 * Convert to a V4 mapped address.
@@ -281,7 +288,9 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 			family = AF_INET;
 			goto common;
 #ifdef LWRES_HAVE_SIN6_SCOPE_ID
-		} else if (ntmp[0] && lwres_net_pton(AF_INET6, ntmp, abuf)) {
+		} else if (ntmp[0] != '\0' &&
+			   lwres_net_pton(AF_INET6, ntmp, abuf) == 1)
+		{
 			if (family && family != AF_INET6)
 				return (EAI_NONAME);
 			addrsize = sizeof(struct in6_addr);
@@ -289,8 +298,8 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 			family = AF_INET6;
 			goto common;
 #endif
-		} else if (lwres_net_pton(AF_INET6, hostname, abuf)) {
-			if (family && family != AF_INET6)
+		} else if (lwres_net_pton(AF_INET6, hostname, abuf) == 1) {
+			if (family != 0 && family != AF_INET6)
 				return (EAI_NONAME);
 		inet6_addr:
 			addrsize = sizeof(struct in6_addr);
@@ -298,7 +307,8 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 			family = AF_INET6;
 
 		common:
-			if ((ai = ai_clone(ai_list, family)) == NULL)
+			ai = ai_clone(ai_list, family);
+			if (ai == NULL)
 				return (EAI_MEMORY);
 			ai_list = ai;
 			ai->ai_socktype = socktype;
@@ -307,7 +317,8 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 			if (flags & AI_CANONNAME) {
 #if defined(LWRES_HAVE_SIN6_SCOPE_ID)
 				if (ai->ai_family == AF_INET6)
-					SIN6(ai->ai_addr)->sin6_scope_id = scopeid;
+					SIN6(ai->ai_addr)->sin6_scope_id =
+									scopeid;
 #endif
 				if (lwres_getnameinfo(ai->ai_addr,
 				    ai->ai_addrlen, nbuf, sizeof(nbuf),
@@ -331,9 +342,10 @@ lwres_getaddrinfo(const char *hostname, const char *servname,
 	for (i = 0; i < FOUND_MAX; i++) {
 		if (net_order[i] == NULL)
 			break;
-		if ((err = (net_order[i])(hostname, flags, &ai_list,
-		    socktype, port)) != 0)
-			return(err);
+		err = (net_order[i])(hostname, flags, &ai_list,
+				     socktype, port);
+		if (err != 0)
+			return (err);
 	}
 
 	if (ai_list == NULL)
@@ -356,13 +368,15 @@ lwres_strsep(char **stringp, const char *delim) {
 	if (string == NULL)
 		return (NULL);
 
-	for (s = string; (sc = *s) != '\0'; s++)
+	for (s = string; *s != '\0'; s++) {
+		sc = *s;
 		for (d = delim; (dc = *d) != '\0'; d++)
 			if (sc == dc) {
 				*s++ = '\0';
 				*stringp = s;
 				return (string);
 			}
+	}
 	*stringp = NULL;
 	return (string);
 }
@@ -438,11 +452,12 @@ add_ipv4(const char *hostname, int flags, struct addrinfo **aip,
 	int result = 0;
 
 	lwres = lwres_context_create(&lwrctx, NULL, NULL, NULL, 0);
-	if (lwres != 0)
+	if (lwres != LWRES_R_SUCCESS)
 		ERR(EAI_FAIL);
-	(void) lwres_conf_parse(lwrctx, "/etc/resolv.conf");
+	(void) lwres_conf_parse(lwrctx, lwres_resolv_conf);
 	if (hostname == NULL && (flags & AI_PASSIVE) == 0) {
-		if ((ai = ai_clone(*aip, AF_INET)) == NULL) {
+		ai = ai_clone(*aip, AF_INET);
+		if (ai == NULL) {
 			lwres_freeaddrinfo(*aip);
 			ERR(EAI_MEMORY);
 		}
@@ -451,8 +466,15 @@ add_ipv4(const char *hostname, int flags, struct addrinfo **aip,
 		ai->ai_socktype = socktype;
 		SIN(ai->ai_addr)->sin_port = port;
 		memcpy(&SIN(ai->ai_addr)->sin_addr, v4_loop, 4);
-	} else if (lwres_getaddrsbyname(lwrctx, hostname,
-					LWRES_ADDRTYPE_V4, &by) == 0) {
+	} else {
+		lwres = lwres_getaddrsbyname(lwrctx, hostname,
+					     LWRES_ADDRTYPE_V4, &by);
+		if (lwres != LWRES_R_SUCCESS) {
+			if (lwres == LWRES_R_NOTFOUND)
+				goto cleanup;
+			else
+				ERR(EAI_FAIL);
+		}
 		addr = LWRES_LIST_HEAD(by->addrs);
 		while (addr != NULL) {
 			ai = ai_clone(*aip, AF_INET);
@@ -480,7 +502,7 @@ add_ipv4(const char *hostname, int flags, struct addrinfo **aip,
 		lwres_conf_clear(lwrctx);
 		lwres_context_destroy(&lwrctx);
 	}
-	return(result);
+	return (result);
 }
 
 static char v6_loop[16] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 };
@@ -497,12 +519,13 @@ add_ipv6(const char *hostname, int flags, struct addrinfo **aip,
 	int result = 0;
 
 	lwres = lwres_context_create(&lwrctx, NULL, NULL, NULL, 0);
-	if (lwres != 0)
+	if (lwres != LWRES_R_SUCCESS)
 		ERR(EAI_FAIL);
-	(void) lwres_conf_parse(lwrctx, "/etc/resolv.conf");
+	(void) lwres_conf_parse(lwrctx, lwres_resolv_conf);
 
 	if (hostname == NULL && (flags & AI_PASSIVE) == 0) {
-		if ((ai = ai_clone(*aip, AF_INET6)) == NULL) {
+		ai = ai_clone(*aip, AF_INET6);
+		if (ai == NULL) {
 			lwres_freeaddrinfo(*aip);
 			ERR(EAI_MEMORY);
 		}
@@ -511,11 +534,19 @@ add_ipv6(const char *hostname, int flags, struct addrinfo **aip,
 		ai->ai_socktype = socktype;
 		SIN6(ai->ai_addr)->sin6_port = port;
 		memcpy(&SIN6(ai->ai_addr)->sin6_addr, v6_loop, 16);
-	} else if (lwres_getaddrsbyname(lwrctx, hostname,
-					LWRES_ADDRTYPE_V6, &by) == 0) {
+	} else {
+		lwres = lwres_getaddrsbyname(lwrctx, hostname,
+					     LWRES_ADDRTYPE_V6, &by);
+		if (lwres != LWRES_R_SUCCESS) {
+			if (lwres == LWRES_R_NOTFOUND)
+				goto cleanup;
+			else
+				ERR(EAI_FAIL);
+		}
 		addr = LWRES_LIST_HEAD(by->addrs);
 		while (addr != NULL) {
-			if ((ai = ai_clone(*aip, AF_INET6)) == NULL) {
+			ai = ai_clone(*aip, AF_INET6);
+			if (ai == NULL) {
 				lwres_freeaddrinfo(*aip);
 				ERR(EAI_MEMORY);
 			}
@@ -548,7 +579,7 @@ lwres_freeaddrinfo(struct addrinfo *ai) {
 
 	while (ai != NULL) {
 		ai_next = ai->ai_next;
-		if (ai->ai_addr)
+		if (ai->ai_addr != NULL)
 			free(ai->ai_addr);
 		if (ai->ai_canonname)
 			free(ai->ai_canonname);
@@ -566,7 +597,8 @@ get_local(const char *name, int socktype, struct addrinfo **res) {
 	if (socktype == 0)
 		return (EAI_SOCKTYPE);
 
-	if ((ai = ai_alloc(AF_LOCAL, sizeof(*sun))) == NULL)
+	ai = ai_alloc(AF_LOCAL, sizeof(*sun));
+	if (ai == NULL)
 		return (EAI_MEMORY);
 
 	sun = SUN(ai->ai_addr);
@@ -590,24 +622,26 @@ get_local(const char *name, int socktype, struct addrinfo **res) {
  *	ai_family
  *	ai_addr
  *	ai_addr->sa_family
- *	ai_addr->sa_len	(HAVE_SA_LEN)
+ *	ai_addr->sa_len	(LWRES_PLATFORM_HAVESALEN)
  * and everything else is initialized to zero.
  */
 static struct addrinfo *
 ai_alloc(int family, int addrlen) {
 	struct addrinfo *ai;
 
-	if ((ai = (struct addrinfo *)calloc(1, sizeof(*ai))) == NULL)
+	ai = (struct addrinfo *)calloc(1, sizeof(*ai));
+	if (ai == NULL)
 		return (NULL);
 
-	if ((ai->ai_addr = SA(calloc(1, addrlen))) == NULL) {
+	ai->ai_addr = SA(calloc(1, addrlen));
+	if (ai->ai_addr == NULL) {
 		free(ai);
 		return (NULL);
 	}
 	ai->ai_addrlen = addrlen;
 	ai->ai_family = family;
 	ai->ai_addr->sa_family = family;
-#ifdef	HAVE_SA_LEN
+#ifdef LWRES_PLATFORM_HAVESALEN
 	ai->ai_addr->sa_len = addrlen;
 #endif
 	return (ai);
@@ -641,7 +675,7 @@ ai_reverse(struct addrinfo *oai) {
 
 	nai = NULL;
 
-	while (oai) {
+	while (oai != NULL) {
 		/*
 		 * Grab one off the old list.
 		 */
